import { ImageType, LqipInline, LqipColor, LqipBlurhash } from '@responsive-image/core';
import { ImageConfig } from 'imagetools-core';
import { Sharp, Metadata } from 'sharp';
import * as url from 'url';

type OutputImageType = 'original' | ImageType;
interface LqipBaseOptions {
    type: string;
}
interface LqipColorOptions extends LqipBaseOptions {
    type: 'color';
}
interface LqipInlineOptions extends LqipBaseOptions {
    type: 'inline';
    targetPixels?: number;
}
interface LqipBlurhashOptions extends LqipBaseOptions {
    type: 'blurhash';
    targetPixels?: number;
}
type LqipOptions = LqipColorOptions | LqipInlineOptions | LqipBlurhashOptions;
interface ImageOptions {
    w: number[];
    quality?: number;
    format: OutputImageType[];
    aspect?: number;
    [key: string]: unknown;
}
interface ImageProcessingResult {
    data: () => Promise<Buffer>;
    width: number;
    format: ImageType;
}
interface ImageLoaderChainedResult {
    lqip?: LqipInline | LqipColor | LqipBlurhash;
    images: ImageProcessingResult[];
    sharp: Sharp;
    imports: string[];
    sharpMeta?: Metadata;
    hash?: string;
}

declare function generateResizedImage(input: ImageLoaderChainedResult, config: ImageConfig, options?: {
    cache?: boolean;
    cacheDir?: string;
}): Promise<ImageProcessingResult>;
declare function effectiveImageFormats(formats: OutputImageType[], meta: Metadata): ImageType[];
declare function getImagetoolsConfigs<BUILDOPTIONS extends Record<string, unknown>>(options: ImageOptions & BUILDOPTIONS, buildKeys: Array<keyof BUILDOPTIONS>): Promise<ImageConfig[]>;

declare function generateLqipClassName(resource: string): string;
declare function blurrySvg(src: string, width: number, height: number): string;

declare const META_KEY = "responsive-image";
declare function parseURL(id: string): url.URL;
declare function getPathname(id: string): string;
declare function parseQuery(query: string | URLSearchParams): Partial<ImageOptions>;
declare function getOptions<BUILDOPTIONS>(id: string | URL, defaultOptions: BUILDOPTIONS, loaderOptions: Partial<BUILDOPTIONS>): ImageOptions & BUILDOPTIONS;
declare function normalizeInput(input: string | Buffer | ImageLoaderChainedResult, options?: {
    generateHash?: boolean;
}): ImageLoaderChainedResult;
declare function parseAspect(aspect: string): number | undefined;
declare function getAspectRatio(meta: Metadata): number | undefined;
declare function onlyUnique<T>(value: T, index: number, self: T[]): boolean;
declare function dataUri(data: string | Buffer, type: string, base64?: boolean): string;
declare function hash(datas: Array<string | object | Buffer>): string;
declare function getCacheFilename(input: ImageLoaderChainedResult, config: ImageConfig, extension: string, cacheDir: string): string;

export { type ImageLoaderChainedResult, type ImageOptions, type ImageProcessingResult, type LqipBaseOptions, type LqipBlurhashOptions, type LqipColorOptions, type LqipInlineOptions, type LqipOptions, META_KEY, type OutputImageType, blurrySvg, dataUri, effectiveImageFormats, generateLqipClassName, generateResizedImage, getAspectRatio, getCacheFilename, getImagetoolsConfigs, getOptions, getPathname, hash, normalizeInput, onlyUnique, parseAspect, parseQuery, parseURL };
